! netcdf_attribute.f90 - object-oriented netCDF interface (attribute)
! vi: set sw=4:
! Copyright (C) TOYODA Eizi, 2000.  All rights reserved.
!
! 2000-01-11 ˭ıѻ	
! 2000-02-22 ˭ıѻ	褦䤯ꤷƥѥǤ褦ˤʤä

module netcdf_attribute
    use iso_varying_string
    use netcdf_error
    use netcdf_filename
    use netcdf_file
    use netcdf_variable
    implicit none

    type NC_ATTRIBUTE
	type(NC_VARIABLE)::		var
	type(VARYING_STRING)::		name
    end type

    interface Attribute
	module procedure NetcdfGlobalAttribute
	module procedure NetcdfVariableAttribute
    end interface

    interface operator(.exists.)
    	module procedure NetcdfAttributeExists
    end interface

    interface Dispose
	module procedure NetcdfAttributeDispose
    end interface

    interface Rename
	module procedure NetcdfAttributeRename
    end interface

    interface Delete
	module procedure NetcdfAttributeDelete
    end interface

    interface Len
	module procedure NetcdfAttributeLength
    end interface

    interface Type
	module procedure NetcdfAttributeType
    end interface

    interface assignment(=)
	module procedure NetcdfAttributePutChar
	module procedure NetcdfAttributePutString
	module procedure NetcdfAttributePutInt
	module procedure NetcdfAttributePutReal
	module procedure NetcdfAttributeCopyVA
	module procedure NetcdfAttributeGetString
	module procedure NetcdfAttributeGetInt
	module procedure NetcdfAttributeGetReal
    end interface

contains

    !
    ! --- 󥹥ȥ饯 ---
    !
    ! 󥹥ȥ饯ƤФ줿°¸ߤ뤫ɤ.
    ! °¸ߤ .exists. 黻ҤǸ
    !

    function NetcdfGlobalAttribute(file, name) result(result)
	type(NC_ATTRIBUTE)::				result
	type(NC_FILE), intent(in)::			file
	character(len = *), intent(in)::		name
    continue
	result%var = NC_VARIABLE(file, NF_GLOBAL)
	result%name = name
    end function

    function NetcdfVariableAttribute(var, name) result(result)
	type(NC_ATTRIBUTE)::				result
	type(NC_VARIABLE), intent(in)::			var
	character(len = *), intent(in)::		name
    continue
	result%var = var
	result%name = name
    end function

    !
    ! --- ǥȥ饯 --- 
    !

    subroutine NetcdfAttributeDispose(attr)
    	type(NC_ATTRIBUTE), intent(inout)::		attr
    continue
	attr%name = ''
    end subroutine

    !
    ! --- ¸ߥå ---
    !

    logical function NetcdfAttributeExists(attr) result(result)
	type(NC_ATTRIBUTE), intent(in)::		attr
	integer::					status, id
    continue
	status = 0
	! char ǤϤʤ chars ؿȤΤ DVF ΥХ
	status = nf_inq_attid(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), id)
	result = (status == NF_NOERR)
    end function

    !
    ! ---  ---
    !
    ! п. ¤ΤѴ

    logical function NetcdfAttributeDelete(attr) result(result)
	type(NC_ATTRIBUTE), intent(inout)::	attr
	integer::				status
    continue
	status = nf_del_att(attr%var%file%id, attr%var%id, &
	    & chars(attr%name))
	call NetcdfSaveError(status, 'AttributeDelete', attr%name)
	result = (status == NF_NOERR)
	! ;פʿ礭ʤ
	call dispose(attr)
    end function

    !
    ! --- ̾ ---
    !
    ! п

    logical function NetcdfAttributeRename(attr, newname) result(result)
	type(NC_ATTRIBUTE), intent(inout)::	attr
	character(len = *)::			newname
	integer::				status
    continue
	status = nf_rename_att(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), newname)
	call NetcdfSaveError(status, 'AttributeDelete', attr%name)
	attr%name = newname
	result = (status == NF_NOERR)
    end function

    !
    ! --- °񤭽Ф ---
    !

    subroutine NetcdfAttributePutInt(attr, value)
	type(NC_ATTRIBUTE), intent(inout)::	attr
	integer, intent(in)::			value(:)
	integer::				status
    continue
	status = nf_put_att_int(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), NF_INT, size(value), value)
	call NetcdfSaveError(status, 'AttributePutInt', attr%name)
    end subroutine

    subroutine NetcdfAttributePutReal(attr, value)
	type(NC_ATTRIBUTE), intent(inout)::	attr
	real, intent(in)::			value(:)
	integer::				status
    continue
	status = nf_put_att_real(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), NF_FLOAT, size(value), value)
	call NetcdfSaveError(status, 'AttributePutReal', attr%name)
    end subroutine

    subroutine NetcdfAttributePutChar(attr, value)
	type(NC_ATTRIBUTE), intent(inout)::	attr
	character(len = *), intent(in)::	value
	integer::				status
    continue
	status = nf_put_att_text(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), len(value), value)
	call NetcdfSaveError(status, 'AttributePutChar', attr%name)
    end subroutine

    subroutine NetcdfAttributePutString(attr, value)
	type(NC_ATTRIBUTE), intent(inout)::	attr
	type(VARYING_STRING), intent(in)::	value
	integer::				status
    continue
	status = nf_put_att_text(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), len(value), char(value))
	call NetcdfSaveError(status, 'AttributePutString', attr%name)
    end subroutine

    !
    ! --- °ž ---
    !

    subroutine NetcdfAttributeCopyVA(lhs, rhs)
	type(NC_VARIABLE), intent(inout)::	lhs
	type(NC_ATTRIBUTE), intent(in)::	rhs
	integer::				status
    continue
	status = nf_copy_att(rhs%var%file%id, rhs%var%id, chars(rhs%name), &
	    & lhs%file%id, lhs%id) 
	call NetcdfSaveError(status, 'AttributeCopyVA', rhs%name)
    end subroutine

    !
    ! --- °˴ط븡 ---
    !

    integer function NetcdfAttributeLength(attr) result(result)
	type(NC_ATTRIBUTE), intent(in)::	attr
	integer::				status
    continue
	status = nf_inq_attlen(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), result)
	call NetcdfSaveError(status, 'AttributeLength', attr%name)
    end function

    integer function NetcdfAttributeType(attr) result(result)
	type(NC_ATTRIBUTE), intent(in)::	attr
	integer::				status
    continue
	status = nf_inq_atttype(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), result)
	call NetcdfSaveError(status, 'AttributeType', attr%name)
    end function

    !
    ! --- °ͤ ---
    !
    ! * դϤǤʤΤǽѿ礭֤
    ! * θϤʤ
    !

    subroutine NetcdfAttributeGetInt(value, attr)
	integer, intent(out)::			value(:)
	type(NC_ATTRIBUTE), intent(in)::	attr
	integer, allocatable::			buffer(:)
	integer::				status, length
    continue
	length = len(attr)
	allocate(buffer(1: length), stat=status)
	if (status /= 0) then
	    call NetcdfSaveError(NF_ENOMEM, 'AttributeGetInt/allocate', &
		& attr%name)
	    return
	endif
	status = nf_get_att_int(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), buffer)
	call NetcdfSaveError(status, 'AttributeGetInt', attr%name)
	length = min(size(buffer), size(value))
	value(1: length) = buffer(1: length)
	deallocate(buffer)
    end subroutine

    subroutine NetcdfAttributeGetReal(value, attr)
	real, intent(out)::			value(:)
	type(NC_ATTRIBUTE), intent(in)::	attr
	real, allocatable::			buffer(:)
	integer::				status, length
    continue
	length = len(attr)
	allocate(buffer(1: length), stat=status)
	if (status /= 0) then
	    call NetcdfSaveError(NF_ENOMEM, 'AttributeGetReal/allocate', &
		& attr%name)
	    return
	endif
	status = nf_get_att_real(attr%var%file%id, attr%var%id, &
	    & chars(attr%name), buffer)
	call NetcdfSaveError(status, 'AttributeGetReal', attr%name)
	length = min(size(buffer), size(value))
	value(1: length) = buffer(1: length)
	deallocate(buffer)
    end subroutine

    subroutine NetcdfAttributeGetString(value, attr)
	type(VARYING_STRING)::			value
	type(NC_ATTRIBUTE), intent(in)::	attr
	integer::				status, length
    continue
	call setbuf(len(attr))
	call NetcdfSaveError(status, 'AttributeGetString', attr%name)
	if (status /= NF_NOERR) value = ''
    contains
	subroutine setbuf(nagasa)
	    integer::				nagasa
	    character(len = nagasa)::		buffer
	continue
	    status = nf_get_att_text(attr%var%file%id, attr%var%id, &
	       & chars(attr%name), buffer)
	    value = buffer
	end subroutine
    end subroutine

end module
